; Core/ARM/3/ARMMMU.s
; ARMMMU module for v3 ARM architectures
;
; 29/Sep/1999 ned

	AREA	ARMMMU, CODE, READONLY

	; Include ARMMMU constants
	INCLUDE ArmMMUH.s

	; Include zero page map
	INCLUDE Core/ARM/CommonH.s

	; Exports from this module
	EXPORT	ARMMMU_Initialise
	EXPORT	ARMMMU_Set
	EXPORT	ARMMMU_SetTranslationTableBase
	EXPORT	ARMMMU_BeginL1PageTableBuild
	EXPORT	ARMMMU_MakeL1PageTableEntries


ARMMMU_Initialise
	BOARD_RETURNZPLOCATION R3
	MOV	R0,#0
	STR	R0,[R3,#HALZP_ARMMMU_TTBREG]
	MOV	PC, R14

ARMMMU_Set
	STMFD	R13!,{R4,R9}
	; R0=bits to set
	; R1=bits to clr
	BOARD_RETURNZPLOCATION R9
	; Ensure interrupts off
	MRS	R4,CPSR
	ORR	R3,R4,#IBit
	MSR	CPSR_all,R3

	LDR	R2,[R9,#HALZP_ARMCORE_CTRLREG]
	BIC	R2,R2,R1
	ORR	R2,R2,R0
	STR	R2,[R9,#HALZP_ARMCORE_CTRLREG]
	MCR	p15, 0, R2, c1, c0, 0

	MSR	CPSR_all,R4
	MOV	R0,#0
	LDMFD	R13!,{R4,R9}
	MOV	PC,R14

ARMMMU_SetTranslationTableBase
	; R0=desired ttb
	BOARD_RETURNZPLOCATION R3
	; Ensure interrupts off
	MRS	R2,CPSR
	ORR	R1,R2,#IBit
	MSR	CPSR_all,R1
	STR	R2,[R9,#HALZP_ARMMMU_TTBREG]
	MCR	p15, 0, R2, c2, c0, 0

	MSR	CPSR_all,R2
	MOV	R0,#0
	MOV	PC,R14

ARMMMU_BeginL1PageTableBuild
	; R0=addr of proposed L1 page tables
	ADD	R1,R0,#16384
	MOV	R2,#0
1
	STR	R2,[R0],#4
	CMP	R0,R1
	BLO	%b01
	MOV	R0,#0
	MOV	PC,R14

ARMMMU_MakeL1PageTableEntries
	; R0=base of L1 page tables
	; R1=start section no
	; R2=start desired virtual addr
	; R3=end desired virtual addr
	; R4=desired domain, access permissions, caching & buffering of section
	STMFD	R13!,{R5}
1
	ORR	R5,R4,R2
	STR	R5,[R0,R1,LSL #2]
	ADD	R2,R2,#0x100000			; +1Mb
	ADD	R1,R1,#1			; Mb no +1
	CMP	R2,R3
	BLO	%b01
	LDMFD	R13!,{R5}
	; R0,R3,R4 preserved
	; R1,R2 updated
	MOV	R0,#0
	MOV	PC,R14

	
	END
